export declare abstract class Handler {
    sparqlEndpointUrl: any;
    sparqlPostprocessor: any;
    language: any;
    searchPath: any;
    listOrder: string;
    constructor(sparqlEndpointUrl: any, sparqlPostprocessor: any, language: any, searchPath: any);
    /**
     * Post-processes the SPARQL query and builds the full URL for list content
     **/
    buildURL(sparql: string): string;
    listLocation(domain: any, property: any, range: any, data: {
        results: {
            bindings: any;
        };
    }): any;
    elementLabel(element: {
        label: {
            value: any;
        };
    }): any;
    elementUri(element: {
        uri: {
            value: any;
        };
        value: {
            value: any;
        };
    }): any;
    enableMatch(domain: any, property: any, range: any): boolean;
}
export declare abstract class AbstractSparqlListHandler extends Handler {
    constructor(sparqlEndpointUrl: any, sparqlPostprocessor: any, language: any, searchPath: any);
    abstract listUrl(domain: string, property: string, range: string): string;
}
export declare abstract class AbstractSparqlAutocompleteHandler extends Handler {
    constructor(sparqlEndpointUrl: any, sparqlPostprocessor: any, language: any, searchPath: any);
    abstract autocompleteUrl(domain: string, property: string, range: string, key: string): string;
}
/**
 * Handles a list widget based on a provided SPARQL query in which
 * $domain, $property and $range will be replaced by actual values.
 **/
export declare class SparqlTemplateListHandler extends AbstractSparqlListHandler {
    queryString: any;
    constructor(sparqlEndpointUrl: any, sparqlPostprocessor: any, language: any, queryString: string);
    /**
     * Constructs the SPARQL query to use for list widget search.
     **/
    listUrl(domain: string, property: string, range: string): string;
}
/**
 * Handles a list widget based on a provided SPARQL query in which
 * $domain, $property and $range will be replaced by actual values.
 **/
export declare class SparqlTemplateAutocompleteHandler extends AbstractSparqlAutocompleteHandler {
    queryString: any;
    constructor(sparqlEndpointUrl: any, sparqlPostprocessor: any, language: any, queryString: any);
    /**
     * Constructs the SPARQL query to use for autocomplete widget search.
     **/
    autocompleteUrl(domain: string, property: string, range: string, key: string): string;
}
export declare class SimpleSparqlAutocompleteAndListHandler extends Handler {
    constructor(sparqlEndpointUrl: any, sparqlPostprocessor: any, language: any, searchPath: any);
    /**
     * Constructs the SPARQL query to use for autocomplete widget search.
     **/
    _buildAutocompleteSparql(domain: any, property: any, range: any, key: any): string;
    /**
     * Constructs the SPARQL query to use for list widget search.
     **/
    _buildListSparql(domain: any, property: any, range: any): string;
}
export declare class UriOnlyListHandler extends Handler {
    constructor(sparqlEndpointUrl: any, sparqlPostprocessor: any);
    /**
     * Constructs the SPARQL query to use for list widget search.
     **/
    _buildListSparql(domain: any, property: any, range: any): string;
}
export declare class SparqlBifContainsAutocompleteAndListHandler extends Handler {
    constructor(sparqlEndpointUrl: any, sparqlPostprocessor: any, language: any, searchPath: any);
    /**
     * Constructs the SPARQL query to use for autocomplete widget search.
     **/
    _buildAutocompleteSparql(domain: any, property: any, range: any, key: any): string;
}
/**
 * Simple handler for GraphDB Lucene connectors : http://graphdb.ontotext.com/documentation/standard/lucene-graphdb-connector.html
 * Takes as input the name of the connector, the field to search on, and the property to read to display
 * TODO : the property to display should come from the snippet
 **/
export declare class GraphDbLuceneConnectorSparqlAutocompleteAndListHandler extends Handler {
    connectorName: any;
    fieldName: any;
    /**
     * The search path, in this case, is used only to read the label,
     * not to search
     **/
    constructor(sparqlEndpointUrl: any, sparqlPostprocessor: any, language: any, searchPath: any, connectorName: any, fieldName: any);
    /**
     * Constructs the SPARQL query to use for autocomplete widget search.
     **/
    _buildAutocompleteSparql(domain: any, property: any, range: any, key: any): string;
}
export declare class WikidataAutocompleteAndListHandler extends Handler {
    constructor(sparqlEndpointUrl: any, sparqlPostprocessor: any, language: any);
    /**
     * Constructs the SPARQL query to use for autocomplete widget search.
     **/
    _buildAutocompleteSparql(domain: any, property: any, range: any, key: any): string;
    _buildListSparql(domain: any, property: any, range: any): string;
}
export declare class RangeBasedAutocompleteAndListHandler {
    defaultHandler: any;
    handlerByKeyMap: any;
    constructor(defaultHandler: any, handlerByKeyMap: any);
    _findHandler(domain: any, property: any, range: string | number): any;
    autocompleteUrl(domain: any, property: any, range: any, key: any): any;
    listUrl(domain: any, property: any, range: any): any;
    listLocation(domain: any, property: any, range: any, data: any): any;
    elementLabel(element: any): any;
    elementUri(element: any): any;
    enableMatch(domain: any, property: any, range: any): any;
}
export declare class PropertyBasedAutocompleteAndListHandler extends RangeBasedAutocompleteAndListHandler {
    constructor(defaultHandler: any, handlerByKeyMap: any);
    _findHandler(domain: any, property: string | number, range: any): any;
}
