import { NamedNode, Quad, Store } from "n3";
import ISpecProvider from "./ISpecProvider";
export declare const RDF: {
    TYPE: NamedNode<string>;
    FIRST: NamedNode<string>;
    REST: NamedNode<string>;
    NIL: NamedNode<string>;
};
export declare const RDFS: {
    LABEL: NamedNode<string>;
    DOMAIN: NamedNode<string>;
    RANGE: NamedNode<string>;
    SUBPROPERTY_OF: NamedNode<string>;
    SUBCLASS_OF: NamedNode<string>;
};
export declare const GEOF: {
    WITHIN: NamedNode<string>;
};
export declare const OWL: {
    EQUIVALENT_PROPERTY: NamedNode<string>;
    EQUIVALENT_CLASS: NamedNode<string>;
    UNION_OF: NamedNode<string>;
};
export declare class OWLSpecificationProvider implements ISpecProvider {
    #private;
    lang: string;
    store: Store<Quad>;
    constructor(n3store: Store<Quad>, lang: string);
    static build(specs: any, filePath: string, lang: any, callback: any): void;
    getAllSparnaturalClasses(): any[];
    getServiceEndpoint(propertyId: string): any;
    isLogicallyExecutedAfter(propertyId: string): boolean;
    getClassesInDomainOfAnyProperty(): any[];
    getLabel(entityId: any): string;
    getTooltip(entityId: any): string;
    getIcon(classId: string): string | any[];
    getHighlightedIcon(classId: string): any[];
    getConnectedClasses(classId: string): any[];
    hasConnectedClasses(classId: any): boolean;
    getConnectingProperties(domainClassId: any, rangeClassId: any): any[];
    getObjectPropertyType(objectPropertyId: any): any;
    isRemoteClass(classUri: string): boolean;
    isLiteralClass(classUri: string): boolean;
    isSparnaturalClass(classUri: string): boolean;
    expandSparql(sparql: string, prefixes: {
        [key: string]: string;
    }): string;
    getDefaultLabelProperty(classId: any): any;
    getBeginDateProperty(propertyId: any): any;
    getEndDateProperty(propertyId: any): any;
    getExactDateProperty(propertyId: any): any;
    getDatasource(propertyOrClassId: any): any;
    getTreeRootsDatasource(propertyOrClassId: any): any;
    getTreeChildrenDatasource(propertyOrClassId: any): any;
    isEnablingOptional(propertyId: any): boolean;
    isEnablingNegation(propertyId: any): boolean;
    isMultilingual(propertyId: any): boolean;
    readRange(propertyId: any): any[];
    _readDatasourceAnnotationProperty(propertyOrClassId: any, datasourceAnnotationProperty: any): any;
    /**
     * {
     *   queryString: "...",
     *   queryTemplate: "...",
     *   labelPath: "...",
     *   labelProperty: "...",
     *   childrenPath: "...",
     *   childrenProperty: "...",
     *   noSort: true
     * }
     **/
    _buildDatasource(datasourceUri: any): {
        queryString?: string;
        queryTemplate?: any;
        labelPath?: any;
        labelProperty?: any;
        childrenPath?: any;
        childrenProperty?: any;
        sparqlEndpointUrl?: any;
        noSort?: any;
    };
    _sort(items: any[]): any[];
    _readPropertiesWithDomain(classId: any): any[];
    _readClassesInRangeOfProperty(propertyId: any): any[];
    _readImmediateSuperClasses(classId: any): any[];
    _readImmediateSubClasses(classId: any): any[];
    /**
     * Reads rdf:type(s) of an entity, and return them as an array
     **/
    _readRdfTypes(uri: any): any[];
    /**
     * Reads config:order of an entity and returns it, or null if not set
     **/
    _readOrder(uri: any): any;
    /**
     * Reads the given property on an entity, and return values as an array
     **/
    _readAsResource(uri: any, property: any): any[];
    /**
     * Reads the given property on an entity, and returns the first value found, or null if not found
     **/
    _readAsSingleResource(uri: any, property: any): any;
    _readAsLiteral(uri: any, property: any): any[];
    _readAsSingleLiteral(uri: any, property: any): any;
    _readAsLiteralWithLang(uri: any, property: any, lang: any, defaultToNoLang?: boolean): string;
    _readAsRdfNode(rdfNode: any, property: any): any[];
    _hasProperty(rdfNode: any, property: any): boolean;
    /*** Handling of UNION classes ***/
    _isUnionClass(classUri: any): boolean;
    _isInUnion(classUri: any): boolean;
    _readUnionContent(classUri: any): any[];
    _readList_rec(list: any): any[];
    _readRootList(listId: any): any;
    _readSuperList(listId: any): string;
    _readUnionsContaining(classId: any): any[];
    /*** / Handling of UNION classes ***/
    _pushIfNotExist(item: any, items: any[]): any[];
}
